﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;
using System.Data.Services.Client;
using DoodleJoyModels;
using System.IO;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace DoodleJoyCloud_Worker
{
    public class WorkerRole : RoleEntryPoint
    {
        public override void Run()
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));

            var queue = account.CreateCloudQueueClient().GetQueueReference("newdoodles");
            queue.CreateIfNotExist();

            var blobs = account.CreateCloudBlobClient();

            while (true)
            {
                var msg = queue.GetMessage();
                if (msg != null)
                {
                    var svc = account.CreateCloudTableClient().GetDataServiceContext();
                    var doodle = svc.CreateQuery<Doodle>("Doodles")
                        .Where(d => d.PartitionKey == string.Empty && d.RowKey == msg.AsString)
                        .Single();
                    var thumbnailUri = GenerateThumbnail(doodle.ImageUri);

                    foreach (var subscription in svc.CreateQuery<Subscription>("Subscriptions").AsTableServiceQuery())
                    {
                        try
                        {
                            SendToastNotification(subscription, doodle);
                            SendTileNotification(subscription, doodle, thumbnailUri);
                        }
                        catch (WebException ex)
                        {
                            if (ex.Status == WebExceptionStatus.ProtocolError && ex.Response != null)
                            {
                                var resp = (HttpWebResponse)ex.Response;
                                if (resp.StatusCode == HttpStatusCode.NotFound)
                                {
                                    // invalid notification URI... stop using it
                                    svc.DeleteObject(subscription);
                                    svc.SaveChangesWithRetries();
                                }
                                else { } // drop the notification, should perhaps retry with backoff
                            }
                            else { throw; } // unexpected
                        }
                    }
                    queue.DeleteMessage(msg);
                }
                else
                {
                    Thread.Sleep(TimeSpan.FromSeconds(1));
                }
            }
        }

        private void SendTileNotification(Subscription subscription, Doodle doodle, string thumbnailUri)
        {
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(subscription.NotificationUri);
            request.Method = "POST";
            request.ContentType = "text/xml";
            request.Headers.Add("X-NotificationClass", "1");
            request.Headers.Add("X-WindowsPhone-Target", "token");

            using (var writer = new StreamWriter(request.GetRequestStream()))
            {
                writer.Write(@"<?xml version=""1.0"" encoding=""utf-8""?>
                <wp:Notification xmlns:wp=""WPNotification"">
                   <wp:Tile>
                      <wp:BackgroundImage>" + thumbnailUri + @"</wp:BackgroundImage>
                      <wp:Count>0</wp:Count>
                      <wp:Title>DoodleJoy</wp:Title>
                   </wp:Tile>
                </wp:Notification>");
            }

            request.GetResponse();
        }

        private void SendToastNotification(Subscription subscription, Doodle doodle)
        {
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(subscription.NotificationUri);
            request.Method = "POST";
            request.ContentType = "text/xml";
            request.Headers.Add("X-NotificationClass", "2");
            request.Headers.Add("X-WindowsPhone-Target", "toast");

            using (var writer = new StreamWriter(request.GetRequestStream()))
            {
                writer.Write(@"<?xml version=""1.0"" encoding=""utf-8""?>
                <wp:Notification xmlns:wp=""WPNotification"">
                   <wp:Toast>
                      <wp:Text1>New doodle</wp:Text1>
                      <wp:Text2>" + doodle.Title + @"</wp:Text2>
                   </wp:Toast>
                </wp:Notification>");
            }

            request.GetResponse();
        }

        private string GenerateThumbnail(string uri)
        {
            Image image;
            var req = (HttpWebRequest)WebRequest.Create(uri);
            var response = req.GetResponse();
            using (var stream = response.GetResponseStream())
            {
                image = Image.FromStream(stream);
            }

            var bmp = new Bitmap(image);
            bmp.MakeTransparent(Color.Black);
            bmp = new Bitmap(bmp, new Size(62, 62));

            var blob = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString")).CreateCloudBlobClient().GetContainerReference("images").GetBlobReference(string.Format("{0}_thumbnail", new Uri(uri).Segments.Last()));
            blob.Properties.ContentType = "image/png";
            var ms = new MemoryStream();
            bmp.Save(ms, ImageFormat.Png);
            ms.Seek(0, SeekOrigin.Begin);
            blob.UploadFromStream(ms);

            //return new UriBuilder(blob.Uri) { Scheme = "http", Port = 80, Host = "cdn.doodlejoy.com", Query = null }.Uri.AbsoluteUri;
            return new UriBuilder(blob.Uri) { Query = null }.Uri.AbsoluteUri;
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            DiagnosticMonitor.Start("DiagnosticsConnectionString");

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            RoleEnvironment.Changing += RoleEnvironmentChanging;

            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }
    }
}
